/* eslint-disable @typescript-eslint/no-var-requires */
/* eslint-disable @typescript-eslint/ban-ts-comment */
// @ts-nocheck
const fs = require('fs');
const axios = require('axios');
const config = JSON.parse(fs.readFileSync('./config.json', 'utf8'));

/**
 * Check if configs are set
 */
const missingConfigKeys = Object.keys(config).filter((key) => config[key] === '');

if (missingConfigKeys.length > 0) {
  console.log(`Some config is missing: ${missingConfigKeys.join(', ')}`);
  process.exit(1);
}

if (
  !config.apiKey ||
  !config.wflogin ||
  !config.wfsession ||
  !config.projectName ||
  !config.languageDomains ||
  (config.languageDomains && config.languageDomains.length === 0)
) {
  console.log(
    `FATAL! Missing config values. Read documentation to see how to configure your config.json file.`
  );
  process.exit(1);
} else {
  console.log('Config is OK');
}

/**
 * Initialize and weglot requests
 */
const wfapi = axios.create({
  baseURL: 'https://webflow.com/',
  headers: {
    Accept: 'application/json, text/javascript, */*; q=0.01',
    'User-Agent':
      'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36',
    Cookie: `wflogin=${config.wflogin};wfsession=${config.wfsession}`,
  },
});

const getCSRF = async () => {
  try {
    const designer = await wfapi.get(`/design/${config.projectName}`).then((res) => res.data);
    console.log(`Searching for CSRF Token...`);

    const matches = designer.match(/<meta name="_csrf" content="([^"]+)">/);
    if (!matches) {
      console.log('Unable to get csrf');
      process.exit();
    }
    const [, csrf] = matches;
    return csrf;
  } catch (_) {
    console.log('Unable to login, please check your config values.');
    process.exit();
  }
};

const getDOM = () => {
  return wfapi
    .get(`/api/sites/${config.projectName}/dom?t=${Date.now()}`, {
      headers: { 'X-Requested-With': 'XMLHttpRequest' },
    })
    .then((res) => {
      return res.data;
    });
};

const updatePage = (pageId, payload, csrf) => {
  return wfapi.put(`/api/pages/${pageId}`, payload, {
    headers: {
      'Content-Type': 'application/json',
      'X-XSRF-Token': csrf,
    },
  });
};

/**
 * Utils
 */
const hreflang = (href, lang) => {
  return `<link href="${href}" hreflang="${lang}" rel="alternate" />`;
};

const getSettings = (apiKey) => {
  return axios
    .get(`https://cdn.weglot.com/projects-settings/${apiKey}.json`)
    .then((res) => res.data);
};

const getSlugs = (apiKey, languages, version) => {
  return languages.map(({ language_to }) =>
    axios
      .get(
        `https://cdn-api.weglot.com/translations/slugs?api_key=${apiKey}&language_to=${language_to}&v=${version}`
      )
      .then((res) => ({ [language_to]: res.data }))
  );
};

function getTranslatedSlug(slug, language) {
  if (!config.slugs || !config.slugs[language]) {
    return slug;
  }
  return config.slugs[language][slug] || slug;
}

function formatSlug(slug, language) {
  if (!slug) {
    return '';
  }
  if (!slug.startsWith('detail_')) {
    return getTranslatedSlug(slug, language);
  }
  // like detail_post, detail_product or detail_category
  const name = getTranslatedSlug(slug.split('detail_').pop(), language);
  const path = config.translateSlugs && language ? `translated-slug-${language}` : 'slug';
  return `${name}/{{wf {&quot;path&quot;:&quot;${path}&quot;,&quot;type&quot;:&quot;PlainText&quot;} }}`;
}

// Recursively construct url pathname with slugs levels
function getSlug(page, language_to) {
  const parent = config.pages.find((p) => p._id === page.parent);
  const slug = formatSlug(page.slug, language_to);
  if (!page.parent || !parent) {
    return `/${slug}`;
  }
  return `${getSlug(parent, language_to)}/${slug}`;
}

// Generate snippet with - JS, - original hreflang, - all translated hreflang versions
function snippet(page) {
  const { languages } = config.settings;
  const defaultPage = config.languageDomains.find((item) => item.defaultPage);
  const currentUrl = `${defaultPage?.domain}${getSlug(page)}`;
  const defaultTag = hreflang(currentUrl, defaultPage?.language) ?? '';
  const defaultTagX = hreflang(currentUrl, 'x-default') ?? '';

  const tags =
    languages.map(({ language_to, custom_code }) => {
      const hrefLangDomain = config.languageDomains.find((item) => {
        if (!item.language) {
          console.log(`Found missing values from your config (LANGUAGE: ${item.language}).
          Please ensure that you have updated the config script with the latest version.
          `);

          process.exit(1);
        }
        return item.language.toLowerCase() === language_to?.toLowerCase();
      });

      return hreflang(
        `${hrefLangDomain?.domain ?? defaultPage?.domain}${getSlug(page, language_to)}`,
        custom_code || (language_to === 'br' ? 'pt-br' : language_to)
      );
    }) ?? '';

  const finalHrefLangTagList = [defaultTag, ...tags, defaultTagX].join('\n');
  return finalHrefLangTagList;
}

/**
 * update hreflang
 */

(async () => {
  // Get Weglot settings
  try {
    config.settings = await getSettings(config.apiKey.split('wg_').pop());
  } catch (_) {
    console.log('Your Weglot API key seems wrong. Please check your config file.');
    process.exit();
  }

  // Get Weglot translated slugs
  if (config.settings.versions) {
    const { slugTranslation = 1 } = config.settings.versions;
    const slugs = await Promise.all(
      getSlugs(config.apiKey, config.settings.languages, slugTranslation)
    ).catch(() => ({}));
    config.slugs = Object.assign(...slugs);
  }

  // Get CSRF
  const csrf = await getCSRF();
  console.log('CSRF found!');

  console.log('\nGet Webflow pages...');
  const dom = await getDOM().catch(() => {
    console.log('Unable to login to your Webflow account, please check wfsession cookie');
    process.exit(1);
  });
  console.log('OK');

  config.pages = dom.pages;

  const cleanupStrings = (str) => str?.trim().replace(/\s/g, '').toLowerCase();

  // update hreflang
  const result = await Promise.all(
    dom.pages.map((page) => {
      const existingHead = cleanupStrings(page.head);

      // we dont need to add scripts so commenting this out
      // const hasWeglot = existingHead?.includes("weglot.initialize");

      // includes <link hreflang="x-default" rel="alternate" />
      const hasHrefLang = existingHead?.includes('hreflang');

      const canUpdateHrefLang = [!hasHrefLang].every((v) => v === true);

      console.log(`\n> ${getSlug(page)} can be updated? `, canUpdateHrefLang ? '✅' : '❌');

      const pageSnippet = snippet(page);

      if (canUpdateHrefLang) {
        console.log(`Updating ${getSlug(page)}...`);
        updatePage(
          page._id,
          JSON.stringify({
            ...page,
            head: page.head ? page.head.concat(pageSnippet, '\n') : pageSnippet,
          }),
          csrf
        )
          .then((res) => {
            console.log(`${res.status === 200 ? 'OK' : 'Failed'} - ${getSlug(page)}`);
          })
          .catch((res) => console.log(res));
        console.log(`OK - ${getSlug(page)} has been updated "✅"`);
        return true;
      }
      console.log(`Reason: hreflang and/or weglot has already been set.\n\n`);
      return false;
    })
  );

  const hasUpdated = result.some((v) => v === true);
  // If any page has been updated, we need to alert the user to publish the site
  if (hasUpdated) {
    console.log(`
    Completed! Pages have been updated!
    ------
    NEXT STEPS: Please check added config on your Webflow design editor <head/> section and publish new version!
    Read Troubleshooting section in README if you have any problem or reach out to Tech support.

    If you note any errors or you page breaks please reach out to Tech support.
    `);
  } else {
    console.log(`
    ------
    Seems like you already have Weglot installed and setup on your Webflow website <head/> section with necessary hreflang tags.
    No update has been made.

    Read Troubleshooting section in README if you have any problem, or this is not the case please contact Tech support
    
    If you note any errors or you page breaks please reach out to Tech support.
    `);
  }
})();
